package dev.fyloz.colorrecipesexplorer.rest

import dev.fyloz.colorrecipesexplorer.model.*
import dev.fyloz.colorrecipesexplorer.service.MaterialService
import org.springframework.http.MediaType
import org.springframework.http.ResponseEntity
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.web.bind.annotation.*
import org.springframework.web.multipart.MultipartFile
import javax.validation.Valid

private const val MATERIAL_CONTROLLER_PATH = "api/material"

@RestController
@RequestMapping(MATERIAL_CONTROLLER_PATH)
@PreAuthorize("hasAuthority('VIEW_MATERIAL')")
class MaterialController(private val materialService: MaterialService) {
    @GetMapping
    fun getAll() =
        ok(materialService.getAll())

    @GetMapping("notmixtype")
    fun getAllNotMixType() =
        ok(materialService.getAllNotMixType())

    @GetMapping("{id}")
    fun getById(@PathVariable id: Long) =
        ok(materialService.getById(id))

    @PostMapping(consumes = [MediaType.MULTIPART_FORM_DATA_VALUE])
    @PreAuthorize("hasAuthority('EDIT_MATERIAL')")
    fun save(@Valid @RequestBody material: MaterialSaveDto, simdutFile: MultipartFile?) =
        created<Material>(MATERIAL_CONTROLLER_PATH) {
            materialService.save(
                materialSaveDto(
                    name = material.name,
                    inventoryQuantity = material.inventoryQuantity,
                    materialTypeId = material.materialTypeId,
                    simdutFile = simdutFile
                )
            )
        }

    @PutMapping(consumes = [MediaType.MULTIPART_FORM_DATA_VALUE])
    @PreAuthorize("hasAuthority('EDIT_MATERIAL')")
    fun update(@Valid @RequestBody material: MaterialUpdateDto, simdutFile: MultipartFile?) =
        noContent {
            materialService.update(
                materialUpdateDto(
                    id = material.id,
                    name = material.name,
                    inventoryQuantity = material.inventoryQuantity,
                    materialTypeId = material.materialTypeId,
                    simdutFile = simdutFile
                )
            )
        }

    @DeleteMapping("{id}")
    @PreAuthorize("hasAuthority('REMOVE_MATERIAL')")
    fun deleteById(@PathVariable id: Long) =
        noContent {
            materialService.deleteById(id)
        }

    @GetMapping("{id}/simdut/exists")
    fun hasSimdut(@PathVariable id: Long) =
        ok(materialService.hasSimdut(id))

    @GetMapping("{id}/simdut", produces = [MediaType.APPLICATION_PDF_VALUE])
    fun getSimdut(@PathVariable id: Long): ResponseEntity<ByteArray> = with(materialService.getSimdut(id)) {
        if (this.isEmpty()) {
            notFound()
        } else {
            ok(this, httpHeaders(contentType = MediaType.APPLICATION_PDF))
        }
    }

    @GetMapping("/simdut")
    fun getAllIdsWithSimdut() =
        ok(materialService.getAllIdsWithSimdut())

    @GetMapping("mix/create/{recipeId}")
    fun getAllForMixCreation(@PathVariable recipeId: Long) =
        ok(materialService.getAllForMixCreation(recipeId))

    @GetMapping("mix/update/{mixId}")
    fun getAllForMixUpdate(@PathVariable mixId: Long) =
        ok(materialService.getAllForMixUpdate(mixId))
}

